/*
 * Copyright [2022] [https://www.baiduc.com]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.baiduc.com
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.baiduc.com
 */
package vip.xiaonuo.erp.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.erp.entity.SysBaseFilterInfo;
import vip.xiaonuo.erp.mapper.SysBaseFilterInfoMapper;
import vip.xiaonuo.erp.param.SysBaseFilterInfoAddParam;
import vip.xiaonuo.erp.param.SysBaseFilterInfoEditParam;
import vip.xiaonuo.erp.param.SysBaseFilterInfoIdParam;
import vip.xiaonuo.erp.param.SysBaseFilterInfoPageParam;
import vip.xiaonuo.erp.service.SysBaseFilterInfoService;

import java.util.List;

/**
 * 保存每个用户的查询选择Service接口实现类
 *
 * @author 1
 * @date  2025/03/01 16:53
 **/
@Service
public class SysBaseFilterInfoServiceImpl extends ServiceImpl<SysBaseFilterInfoMapper, SysBaseFilterInfo> implements SysBaseFilterInfoService {

    @Override
    public Page<SysBaseFilterInfo> page(SysBaseFilterInfoPageParam sysBaseFilterInfoPageParam) {
        QueryWrapper<SysBaseFilterInfo> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(sysBaseFilterInfoPageParam.getSortField(), sysBaseFilterInfoPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(sysBaseFilterInfoPageParam.getSortOrder());
            queryWrapper.orderBy(true, sysBaseFilterInfoPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(sysBaseFilterInfoPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(SysBaseFilterInfo::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(SysBaseFilterInfoAddParam sysBaseFilterInfoAddParam) {
        SysBaseFilterInfo sysBaseFilterInfo = BeanUtil.toBean(sysBaseFilterInfoAddParam, SysBaseFilterInfo.class);
        this.save(sysBaseFilterInfo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(SysBaseFilterInfoEditParam sysBaseFilterInfoEditParam) {
        SysBaseFilterInfo sysBaseFilterInfo = this.queryEntity(sysBaseFilterInfoEditParam.getId());
        BeanUtil.copyProperties(sysBaseFilterInfoEditParam, sysBaseFilterInfo);
        this.updateById(sysBaseFilterInfo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<SysBaseFilterInfoIdParam> sysBaseFilterInfoIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(sysBaseFilterInfoIdParamList, SysBaseFilterInfoIdParam::getId));
    }

    @Override
    public SysBaseFilterInfo detail(SysBaseFilterInfoIdParam sysBaseFilterInfoIdParam) {
        return this.queryEntity(sysBaseFilterInfoIdParam.getId());
    }

    @Override
    public SysBaseFilterInfo queryEntity(Integer id) {
        SysBaseFilterInfo sysBaseFilterInfo = this.getById(id);
        if(ObjectUtil.isEmpty(sysBaseFilterInfo)) {
            throw new CommonException("保存每个用户的查询选择不存在，id值为：{}", id);
        }
        return sysBaseFilterInfo;
    }
}
