/*
 * Copyright [2022] [https://www.baiduc.com]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.baiduc.com
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.baiduc.com
 */
package vip.xiaonuo.hr.modular.basedepartment.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.hr.modular.basedepartment.entity.HrBaseDepartment;
import vip.xiaonuo.hr.modular.basedepartment.mapper.HrBaseDepartmentMapper;
import vip.xiaonuo.hr.modular.basedepartment.param.HrBaseDepartmentAddParam;
import vip.xiaonuo.hr.modular.basedepartment.param.HrBaseDepartmentEditParam;
import vip.xiaonuo.hr.modular.basedepartment.param.HrBaseDepartmentIdParam;
import vip.xiaonuo.hr.modular.basedepartment.param.HrBaseDepartmentPageParam;
import vip.xiaonuo.hr.modular.basedepartment.service.HrBaseDepartmentService;


import java.util.List;

/**
 * 人员部门Service接口实现类
 *
 * @author 余
 * @date  2024/12/16 11:09
 **/
@Service
public class HrBaseDepartmentServiceImpl extends ServiceImpl<HrBaseDepartmentMapper, HrBaseDepartment> implements HrBaseDepartmentService {

    @Override
    public Page<HrBaseDepartment> page(HrBaseDepartmentPageParam hrBaseDepartmentPageParam) {
        QueryWrapper<HrBaseDepartment> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(hrBaseDepartmentPageParam.getSortField(), hrBaseDepartmentPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(hrBaseDepartmentPageParam.getSortOrder());
            queryWrapper.orderBy(true, hrBaseDepartmentPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(hrBaseDepartmentPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(HrBaseDepartment::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(HrBaseDepartmentAddParam hrBaseDepartmentAddParam) {
        HrBaseDepartment hrBaseDepartment = BeanUtil.toBean(hrBaseDepartmentAddParam, HrBaseDepartment.class);
        this.save(hrBaseDepartment);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(HrBaseDepartmentEditParam hrBaseDepartmentEditParam) {
        HrBaseDepartment hrBaseDepartment = this.queryEntity(hrBaseDepartmentEditParam.getId());
        BeanUtil.copyProperties(hrBaseDepartmentEditParam, hrBaseDepartment);
        this.updateById(hrBaseDepartment);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<HrBaseDepartmentIdParam> hrBaseDepartmentIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(hrBaseDepartmentIdParamList, HrBaseDepartmentIdParam::getId));
    }

    @Override
    public HrBaseDepartment detail(HrBaseDepartmentIdParam hrBaseDepartmentIdParam) {
        return this.queryEntity(hrBaseDepartmentIdParam.getId());
    }

    @Override
    public HrBaseDepartment queryEntity(String id) {
        HrBaseDepartment hrBaseDepartment = this.getById(id);
        if(ObjectUtil.isEmpty(hrBaseDepartment)) {
            throw new CommonException("人员部门不存在，id值为：{}", id);
        }
        return hrBaseDepartment;
    }
}
